STOP_WORDS = set(
    """
ага адам айтты айтымында айтып ал алар
алардын алган алуу алып анда андан аны
анын ар

бар басма баш башка башкы башчысы берген
биз билдирген билдирди бир биринчи бирок
бишкек болгон болот болсо болуп боюнча
буга бул

гана

да дагы деген деди деп

жана жатат жаткан жаңы же жогорку жок жол
жолу

кабыл калган кандай карата каршы катары
келген керек кийин кол кылмыш кыргыз
күнү көп

маалымат мамлекеттик мен менен миң
мурдагы мыйзам мындай мүмкүн

ошол ошондой

сүрөт сөз

тарабынан турган тууралуу

укук учурда

чейин чек

экенин эки эл эле эмес эми эч

үч үчүн

өз
""".split()
)
